#include <stdio.h>

	// define the cuda kernal
	// this code executed by every thread

__global__ void kernel0(float* x, float* results)
{

	//the 2D arrays of x,v,f with M rows and N columns are the same as 1D of size M*N
	//access as data[y*N+x] where x and y are the column- and row-indices 

	// identify each thread
	int tid = threadIdx.x + blockDim.x*blockIdx.x;
	
	// do some calculations
	// e.g., each thread can access an array element

	results[tid] = x[tid];
	__syncthreads();

	// do something else ...

}

float* callcuda(int natoms, float* x)
{
	// setup threads and blocks
	int nthreads=256; // threads per block
	int nblocks= (int)(natoms*3/nthreads)+1; // num of blocks
	int numthreads=nblocks*nthreads; // total num of threads

	printf("\nnumthreads: %i\n", numthreads);

	// allocate for coordinates and results on device
	// add some padding since numthreads>natoms
	float* dev_ptr_x;
	float* results_dev;
	cudaMalloc((void **)&dev_ptr_x, numthreads*3*sizeof(float));
	cudaMalloc((void **)&results_dev, numthreads*3*sizeof(float));

	// copy from host to device
	// host=cpu, device=gpu
	cudaMemcpy((void*)dev_ptr_x, (const void*)x, natoms*3*sizeof(float),cudaMemcpyHostToDevice);

	// call kernel 
	kernel0<<<nblocks,nthreads>>>(dev_ptr_x, results_dev); // add other kernals if you want

	// allocate for results on host
	float* results = (float*)malloc(numthreads*3*sizeof(float));

	// copy from device to host
	cudaMemcpy(results, results_dev, natoms*3*sizeof(float), cudaMemcpyDeviceToHost);

	// free dev pointers
	cudaFree(dev_ptr_x);
	cudaFree(results_dev);

	// return a pointer to the results on host
	return results;

}

